@@ -31,22 +31,24 @@ if TYPE_CHECKING:
 __all__ = ['CommandParameters']
 
 OptionMap = Dict[str, BaseOption]
+ActionFlags = List[ActionFlag]
 
 
 class CommandParameters:
-    command: CommandType                            #: The Command associated with this CommandParameters object
-    formatter: CommandHelpFormatter                 #: The formatter used for this Command's help text
-    command_parent: Optional[CommandType] = None    #: The parent Command, if any
-    parent: Optional[CommandParameters] = None      #: The parent Command's CommandParameters
-    action: Optional[Action] = None                 #: An Action Parameter, if specified
-    _pass_thru: Optional[PassThru] = None           #: A PassThru Parameter, if specified
-    sub_command: Optional[SubCommand] = None        #: A SubCommand Parameter, if specified
-    action_flags: List[ActionFlag]                  #: List of action flags
-    options: List[BaseOption]                       #: List of optional Parameters
-    combo_option_map: OptionMap                     #: Mapping of {short opt: Parameter} (no dash characters)
-    groups: List[ParamGroup]                        #: List of ParamGroup objects
-    positionals: List[BasePositional]               #: List of positional Parameters
-    option_map: OptionMap                           #: Mapping of {--opt / -opt: Parameter}
+    command: CommandType                                 #: The Command associated with this CommandParameters object
+    formatter: CommandHelpFormatter                      #: The formatter used for this Command's help text
+    command_parent: Optional[CommandType] = None         #: The parent Command, if any
+    parent: Optional[CommandParameters] = None           #: The parent Command's CommandParameters
+    action: Optional[Action] = None                      #: An Action Parameter, if specified
+    _pass_thru: Optional[PassThru] = None                #: A PassThru Parameter, if specified
+    sub_command: Optional[SubCommand] = None             #: A SubCommand Parameter, if specified
+    action_flags: ActionFlags                            #: List of action flags
+    split_action_flags: Tuple[ActionFlags, ActionFlags]  #: Action flags split by before/after main
+    options: List[BaseOption]                            #: List of optional Parameters
+    combo_option_map: OptionMap                          #: Mapping of {short opt: Parameter} (no dash characters)
+    groups: List[ParamGroup]                             #: List of ParamGroup objects
+    positionals: List[BasePositional]                    #: List of positional Parameters
+    option_map: OptionMap                                #: Mapping of {--opt / -opt: Parameter}
 
     def __init__(self, command: CommandType, command_parent: Optional[CommandType], config: CommandConfig):
         self.command = command
@@ -235,9 +237,16 @@ class CommandParameters:
                 raise ParameterDefinitionError(f'No function was registered for param={param!r}')
             grouped_ordered_flags[param.before_main][param.order].append(param)
 
+        found_non_always = False
         invalid = {}
-        for before_main, prio_params in grouped_ordered_flags.items():  # noqa  # pycharm forgets dict has .items...
-            for prio, params in prio_params.items():  # noqa
+        for before_main, prio_params in grouped_ordered_flags.items():
+            for prio, params in prio_params.items():
+                param: ActionFlag = params[0]  # Don't pop and check `if params` - all are needed for the group check
+                if found_non_always and param.always_available:
+                    invalid[(before_main, prio)] = param
+                elif not param.always_available:
+                    found_non_always = True
+
                 if len(params) > 1:
                     group = next((p.group for p in params if p.group), None)
                     if group and group.mutually_exclusive:
@@ -249,10 +258,13 @@ class CommandParameters:
         if invalid:
             raise CommandDefinitionError(
                 f'ActionFlag parameters with the same before/after main setting must either have different order values'
-                f' or be in a mutually exclusive ParamGroup - invalid parameters: {invalid}'
+                f' or be in a mutually exclusive ParamGroup, and always_available ActionFlags must all be ordered'
+                f' before any that are not always available - invalid parameters: {invalid}'
             )
 
+        n_before = len(grouped_ordered_flags[True])
         self.action_flags = action_flags
+        self.split_action_flags = action_flags[:n_before], action_flags[n_before:]
 
     # endregion
 
