@@ -13,22 +13,17 @@ from .base import InputType, TypeFunc
 from .choices import Choices, ChoiceMap, EnumChoices
 from .files import Path, File, Serialized, Json, Pickle
 from .numeric import Range, NumRange
+from .time import Day, Month, DateTime, Date, Time, DTFormatMode
 
+# fmt: off
 __all__ = [
-    'StatMode',
-    'FileWrapper',
-    'Path',
-    'File',
-    'Serialized',
-    'Json',
-    'Pickle',
-    'Range',
-    'NumRange',
-    'Choices',
-    'ChoiceMap',
-    'EnumChoices',
+    'StatMode', 'FileWrapper', 'Path', 'File', 'Serialized', 'Json', 'Pickle',
+    'Range', 'NumRange',
+    'Choices', 'ChoiceMap', 'EnumChoices',
+    'Day', 'Month', 'DateTime', 'Date', 'Time', 'DTFormatMode',
     'normalize_input_type',
 ]
+# fmt: on
 
 InputTypeFunc = _t.Union[None, TypeFunc, InputType, range, _t.Type[_Enum]]
 
@@ -38,6 +33,8 @@ def normalize_input_type(
 ) -> _t.Optional[TypeFunc]:
     param_choices_provided = param_choices is not None
 
+    # TODO: If choices=range(0, 100) is used, for example, that should be turned into a Range type input
+    # TODO: Test help text with range choices/type
     if param_choices_provided:
         if isinstance(param_choices, range):
             return Range(param_choices, type_func)
