@@ -53,7 +53,7 @@ class Path(InputType):
         self.exists = exists
         self.expand = expand
         self.resolve = resolve
-        self.type = StatMode(type)
+        self.type = StatMode(type)  # pylint: disable=E1120
         self.readable = readable
         self.writable = writable
         self.allow_dash = allow_dash
