@@ -3,17 +3,16 @@ Base classes and helpers for Parameters and Groups
 
 :author: Doug Skrypa
 """
+# pylint: disable=R0801
 
 from __future__ import annotations
 
-# import logging
 import re
 from abc import ABC, abstractmethod
-from functools import partial, update_wrapper, reduce
+from contextvars import ContextVar
+from functools import partial, update_wrapper
 from itertools import chain
-from operator import xor
-from typing import TYPE_CHECKING, Any, Type, Optional, Callable, Collection, Union, List, Set, FrozenSet
-from types import MethodType
+from typing import TYPE_CHECKING, Any, Type, Optional, Callable, Collection, Union, List, FrozenSet
 
 try:
     from functools import cached_property  # pylint: disable=C0412
@@ -28,14 +27,18 @@ from ..inputs import InputType, normalize_input_type, Choices, ChoiceMap as Choi
 from ..inputs.exceptions import InputValidationError, InvalidChoiceError
 from ..nargs import Nargs
 from ..utils import _NotSet, Bool, get_descriptor_value_type
+from .option_strings import OptionStrings
 
 if TYPE_CHECKING:
+    from types import MethodType
     from ..core import CommandType
     from ..commands import Command
     from ..formatting.params import ParamHelpFormatter
+    from .groups import ParamGroup
 
 __all__ = ['Parameter', 'BasePositional', 'BaseOption']
-# log = logging.getLogger(__name__)
+
+_group_stack = ContextVar('cli_command_parser.parameters.base.group_stack', default=[])
 
 
 class parameter_action:  # pylint: disable=C0103
@@ -103,7 +106,7 @@ class ParamBase(ABC):
         self.name = name
         self.help = help
         self.hide = hide
-        group = ParamGroup.active_group()
+        group = get_active_param_group()
         if group:
             group.register(self)  # noqa  # This sets self.group = group
 
@@ -126,7 +129,7 @@ class ParamBase(ABC):
         self.__name = name
 
     def __hash__(self) -> int:
-        return reduce(xor, map(hash, (self.__class__, self.__name, self.name, self.command)))
+        return hash(self.__class__) ^ hash(self.__name) ^ hash(self.name) ^ hash(self.command)
 
     def _ctx(self, command: Command = None) -> Context:
         try:
@@ -323,7 +326,6 @@ class Parameter(ParamBase, ABC):
     def take_action(  # pylint: disable=W0613
         self, value: Optional[str], short_combo: bool = False, opt_str: str = None
     ):
-        # log.debug(f'{self!r}.take_action({value!r})')
         action = self.action
         if action == 'append' and self._nargs_max_reached():
             val_count = len(ctx.get_parsing_value(self))
@@ -548,43 +550,24 @@ class BaseOption(Parameter, ABC):
     :param kwargs: Additional keyword arguments to pass to :class:`Parameter`.
     """
 
-    _long_opts: Set[str]                        # --long options
-    _short_opts: Set[str]                       # -short options
-    short_combinable: Set[str]                  # short options without the leading dash (for combined flags)
-    name_mode: Optional[OptionNameMode] = None  # OptionNameMode override
+    _opt_str_cls = OptionStrings
+    option_strs: OptionStrings
 
     def __init__(self, *option_strs: str, action: str, name_mode: Union[OptionNameMode, str] = None, **kwargs):
         _validate_opt_strs(option_strs)
         super().__init__(action, **kwargs)
-        self._long_opts = {opt for opt in option_strs if opt.startswith('--')}
-        self._short_opts = short_opts = {opt for opt in option_strs if 1 == opt.count('-', 0, 2)}
-        self.short_combinable = {opt[1:] for opt in short_opts if len(opt) == 2}
-        if name_mode is not None:
-            self.name_mode = OptionNameMode(name_mode)
-        bad_opts = ', '.join(opt for opt in short_opts if '-' in opt[1:])
-        if bad_opts:
-            raise ParameterDefinitionError(f"Bad short option(s) - may not contain '-': {bad_opts}")
+        self.option_strs = self._opt_str_cls(option_strs, name_mode)
 
     def __set_name__(self, command: CommandType, name: str):
         super().__set_name__(command, name)
-        if not self._long_opts:
-            mode = self.name_mode if self.name_mode is not None else self._config(command).option_name_mode
-            if mode & OptionNameMode.DASH:
-                self._long_opts.add('--{}'.format(name.replace('_', '-')))
-            if mode & OptionNameMode.UNDERSCORE:
-                self._long_opts.add(f'--{name}')
-            try:
-                del self.__dict__['long_opts']
-            except KeyError:
-                pass
+        self.option_strs.update(self, command, name)
 
-    @cached_property
-    def long_opts(self) -> List[str]:
-        return sorted(self._long_opts, key=lambda opt: (-len(opt), opt))
 
-    @cached_property
-    def short_opts(self) -> List[str]:
-        return sorted(self._short_opts, key=lambda opt: (-len(opt), opt))
+def get_active_param_group() -> Optional[ParamGroup]:
+    try:
+        return _group_stack.get()[-1]
+    except (AttributeError, IndexError):
+        return None
 
 
 def _validate_opt_strs(opt_strs: Collection[str]):
@@ -600,7 +583,3 @@ def _is_numeric(text: str) -> Bool:
     except AttributeError:
         _is_numeric._num_match = num_match = re.compile(r'^-\d+$|^-\d*\.\d+?$').match
     return num_match(text)
-
-
-# Down here due to circular dependency
-from .groups import ParamGroup  # pylint: disable=C0413
