@@ -8,21 +8,27 @@ from __future__ import annotations
 
 from functools import partial, update_wrapper, reduce
 from operator import xor
-from typing import TYPE_CHECKING, Any, Optional, Callable, Union, Tuple
+from typing import TYPE_CHECKING, Any, Optional, Callable, Union, Iterator, Tuple, List, Set
+
+try:
+    from functools import cached_property  # pylint: disable=C0412
+except ImportError:
+    from ..compat import cached_property
 
-from ..config import OptionNameMode
 from ..context import ctx
 from ..exceptions import ParameterDefinitionError, BadArgument, CommandDefinitionError, ParamUsageError
 from ..inputs import InputTypeFunc, normalize_input_type
 from ..nargs import Nargs, NargsValue
 from ..utils import _NotSet, Bool
 from .base import BasicActionMixin, BaseOption, parameter_action
+from .option_strings import TriFlagOptionStrings
 
 if TYPE_CHECKING:
     from ..core import CommandType
     from ..commands import Command
 
 __all__ = ['Option', 'Flag', 'TriFlag', 'ActionFlag', 'Counter', 'action_flag', 'before_main', 'after_main']
+# TODO: envvar param to pull value from an env var (or tuple of vars, in order) if no value given, but env var is set?
 
 
 class Option(BasicActionMixin, BaseOption):
@@ -83,6 +89,13 @@ class _Flag(BaseOption):
         super().__init_subclass__(**kwargs)
         cls._use_opt_str = use_opt_str
 
+    def __init__(self, *option_strs: str, **kwargs):
+        bad = ', '.join(repr(key) for key in ('choices', 'metavar') if key in kwargs)
+        if bad:
+            art, s = ('', 's') if ',' in bad else ('an ', '')
+            raise TypeError(f'{self.__class__.__name__}.__init__() got {art}unexpected keyword argument{s}: {bad}')
+        super().__init__(*option_strs, **kwargs)
+
     def _init_value_factory(self):  # pylint: disable=W0221
         if self.action == 'store_const':
             return self.default
@@ -128,8 +141,6 @@ class Flag(_Flag, accepts_values=False, accepts_none=True):
     def __init__(
         self, *option_strs: str, action: str = 'store_const', default: Any = _NotSet, const: Any = _NotSet, **kwargs
     ):
-        if 'choices' in kwargs:
-            raise TypeError(f"{self.__class__.__name__}.__init__() got an unexpected keyword argument 'choices'")
         if const is _NotSet:
             try:
                 const = self.__default_const_map[default]
@@ -162,7 +173,7 @@ class TriFlag(_Flag, accepts_values=False, accepts_none=True, use_opt_str=True):
     :param alt_prefix: The prefix to add to the assigned name for the alternate long form.  Ignored if ``alt_long`` is
       specified.  Defaults to ``no`` if ``alt_long`` is not specified.
     :param alt_long: The alternate long form to use.
-    :param alt_short: The alternate short form to use
+    :param alt_short: The alternate short form to use.
     :param action: The action to take on individual parsed values.  Only ``store_const`` (the default) is supported.
     :param default: The default value to use if neither the primary or alternate options are provided.  Defaults
       to None.
@@ -170,7 +181,8 @@ class TriFlag(_Flag, accepts_values=False, accepts_none=True, use_opt_str=True):
     :param kwargs: Additional keyword arguments to pass to :class:`.BaseOption`.
     """
 
-    alt_short = ()
+    _opt_str_cls = TriFlagOptionStrings
+    option_strs: TriFlagOptionStrings
 
     def __init__(
         self,
@@ -183,9 +195,7 @@ class TriFlag(_Flag, accepts_values=False, accepts_none=True, use_opt_str=True):
         default: Any = None,
         **kwargs,
     ):
-        if 'choices' in kwargs:
-            raise TypeError(f"{self.__class__.__name__}.__init__() got an unexpected keyword argument 'choices'")
-        elif alt_short and '-' in alt_short[1:]:
+        if alt_short and '-' in alt_short[1:]:
             raise ParameterDefinitionError(f"Bad alt_short option - may not contain '-': {alt_short}")
         elif alt_prefix and ('=' in alt_prefix or alt_prefix.startswith('-')):
             raise ParameterDefinitionError(f"Bad alt_prefix - may not contain '=' or start with '-': {alt_prefix}")
@@ -201,30 +211,15 @@ class TriFlag(_Flag, accepts_values=False, accepts_none=True, use_opt_str=True):
         alt_opt_strs = tuple(filter(None, (alt_short, alt_long)))
         super().__init__(*option_strs, *alt_opt_strs, action=action, default=default, **kwargs)
         self.consts = consts
-        self._alt_prefix = alt_prefix
-        self.alt_long = (alt_long,) if alt_long else set()
-        if alt_short:
-            self.alt_short = {alt_short, alt_short[1:]}
-        # TODO: Add formatter for this to separate / properly indicate primary/alternate results
+        self.option_strs.add_alts(alt_prefix, alt_long, alt_short)
 
     def __set_name__(self, command: CommandType, name: str):
         super().__set_name__(command, name)
-        if not self.alt_long:
-            mode = self.name_mode if self.name_mode is not None else self._config(command).option_name_mode
-            if mode & OptionNameMode.DASH:
-                self.alt_long.add('--{}-{}'.format(self._alt_prefix, name.replace('_', '-')))
-            if mode & OptionNameMode.UNDERSCORE:
-                self.alt_long.add(f'--{self._alt_prefix}_{name}')
-
-            self._long_opts.update(self.alt_long)
-            try:
-                del self.__dict__['long_opts']
-            except KeyError:
-                pass
+        self.option_strs.update_alts(self, command, name)
 
     @parameter_action
     def store_const(self, opt_str: str):
-        if opt_str in self.alt_long or opt_str in self.alt_short:
+        if opt_str in self.option_strs.alt_allowed:
             const = self.consts[1]
         else:
             const = self.consts[0]
@@ -379,7 +374,7 @@ class Counter(BaseOption, accepts_values=True, accepts_none=True):
         try:
             return self.type(value)
         except (ValueError, TypeError) as e:
-            combinable = self.short_combinable
+            combinable = self.option_strs.combinable
             if short_combo and combinable and all(c in combinable for c in value):
                 return len(value) + 1  # +1 for the -short that preceded this value
             raise BadArgument(self, f'bad counter value={value!r}') from e
