@@ -4,6 +4,8 @@ Utilities for extracting types from annotations, finding / storing program metad
 :author: Doug Skrypa
 """
 
+from __future__ import annotations
+
 from collections.abc import Collection, Iterable
 from enum import Flag
 from inspect import isclass
@@ -109,6 +111,17 @@ def _type_from_collection(origin, annotation) -> Optional[type]:
 # endregion
 
 
+class MissingMixin:
+    @classmethod
+    def _missing_(cls, value):
+        if isinstance(value, str):
+            try:
+                return cls._member_map_[value.upper()]  # noqa
+            except KeyError:
+                pass
+        return super()._missing_(value)  # noqa
+
+
 class FixedFlag(Flag):
     """Extends Flag to work around breaking changes in 3.11 for repr, missing, and pseudo-members."""
 
@@ -166,6 +179,8 @@ class FixedFlag(Flag):
 
 
 class Terminal:  # pylint: disable=R0903
+    __slots__ = ('_cache_time', '_last_time', '_width')
+
     def __init__(self, cache_time: float = 1):
         self._cache_time = cache_time
         self._last_time = 0
